;; Join arguments by directly pasting them together.
(define (augmk/paste . args)
  (string-join args ""))

;; Emit a variable assignment.
(define (augmk/emit-assign var value)
  (gmk-eval (augmk/paste var ":=" value)))

;; Add targets defined in subdir/Rules.mk.
(define (augmk/enter subdir)
  (let*
      (;; Save the current full path and compute the new full path.
       (cur-fpath (gmk-expand "$(augmk_d)"))
       (new-fpath
        (if (string-null? cur-fpath)
            subdir
            (augmk/paste cur-fpath "/" subdir))))
    ;; Set augmk_d to have the new full path.
    (augmk/emit-assign "augmk_d" new-fpath)
    ;; Include the subdirectory rules.mk.
    (gmk-eval "include $(augmk_d)/Rules.mk")
    ;; Set augmk_d to have the previous full path.
    (augmk/emit-assign "augmk_d" cur-fpath)))

(define augmk/clean-target-template "
augmk_name := ~a

augmk_group_name := ~a
augmk_group := $($(augmk_group_name))

augmk_phonies := $(foreach elt,$(augmk_group),$(elt)_phony)

# All targets are present here.
#$(info $(augmk_phonies))
#$(info $(augmk_phonies:_phony=))

.PHONY: $(augmk_phonies)
$(augmk_phonies):
	rm -rf $(@:_phony=)

.PHONY: $(augmk_name)
$(augmk_name): $(augmk_phonies)
")

;; ^^ Is this a Make bug?

;; Create a clean target from a target group.
;;
;; Clean targets have special support because they create one phony deletion
;; target per member in the specified target group.  Then, they emit the named
;; target with a dependency on all of the created phonies.
(define (augmk/create-clean-target name group)
  (gmk-eval (format #f augmk/clean-target-template name group)))


